#+TITLE:     Getting Things Done With Gnus
#+AUTHOR:    Ryan Rix
#+EMAIL:     ryan@whatthefuck.computer
#+DATE:      2015-04-29
#+DESCRIPTION:
#+KEYWORDS:
#+LANGUAGE:  en
#+OPTIONS:   H:3 num:t toc:t \n:nil @:t ::t |:t ^:t -:t f:t *:t <:t
#+OPTIONS:   TeX:t LaTeX:t skip:nil d:nil todo:t pri:nil tags:not-in-toc
#+INFOJS_OPT: view:nil toc:nil ltoc:t mouse:underline buttons:0 path:http://orgmode.org/org-info.js
#+EXPORT_SELECT_TAGS: export
#+EXPORT_EXCLUDE_TAGS: noexport
#+LINK_UP:
#+LINK_HOME:
#+BEAMER_THEME: default
#+LATEX_CLASS: beamer
#+LATEX_CLASS_OPTIONS: [presentation]
#+COLUMNS: %45ITEM %10BEAMER_ENV(Env) %10BEAMER_ACT(Act) %4BEAMER_COL(Col) %8BEAMER_OPT(Opt)
#+BEAMER_FRAME_LEVEL: 3

* Why Use Gnus?

- Integration: All of my Emacs tools are available
- Automation: Score, filter, archive and delete my mail
- Single point of information: Consume all sorts of information sources and present them in a unified interface
  - nntp (gmane, etc)
  - IMAP
  - POP
  - Maildir, mbox
  - RSS

* How Do I Get Gnus?

It's built right in to your favorite Emacs distribution.

=M-x gnus=

0* Basic Setup

#+begin_src emacs-lisp
(setq user-mail-address "ry@n.rix.si")
(setq user-full-name    "Ryan Rix")
#+end_src

I use =gnutls= instead of the built in SSL stuff because it's quite difficult to get self-signed certificates to behave with them; gnutls has TOFU which makes it possible to globally trust self-signed certificates while keeping the clients stateless with regards to that.

#+begin_src emacs-lisp
(if (fboundp 'gnutls-available-p)
    (fmakunbound 'gnutls-available-p))
(setq tls-program '("gnutls-cli --tofu -p %p %h")
      starttls-extra-arguments '("--tofu")
      imap-ssl-program '("gnutls-cli --tofu -p %p %s"))
#+end_src

* Basic Setup

Strip out duplicate messages; splitting sometimes can cause this.

#+begin_src emacs-lisp
(setq gnus-suppress-duplicates t
      gnus-summary-ignore-duplicates t)
#+end_src

* Gnus For IMAP

- Open up =~/.gnus.el=

#+begin_src emacs-lisp
(setq gnus-select-method '(nnimap "fastmail"
          (nnimap-server-port 993)
          (nnimap-address "mail.messagingengine.com")
          (nnimap-stream ssl)
          (nnir-search-engine imap)
          (nnimap-authinfo-file "~/.authinfo.gpg")
          (nnimap-list-pattern ("INBOX"))
          (nnimap-inbox ("INBOX"))))
#+end_src

- We have a single IMAP account, hit =R= in the =*Group*= to restart Gnus

* Sending Emails and Postings

I use [[http://msmtp.sourceforge.net/][msmtp]] to handle outgoing email, you should, too.

- Multiple account support
- Easy to configure
- Supports TLS/SSL

#+begin_src shell
su -c "yum install msmtp"
#+end_src

* Sending Emails and Postings (~/.msmtprc)

#+begin_example
defaults
tls on
tls_certcheck off
tls_starttls on

##########################
account fastmail
host mail.messagingengine.com
tls on
tls_certcheck off
tls_starttls on
auth on
user rrix@fastmail.com
from ryan@whatthefuck.computer
password [REDACTED]
port 587
logfile /tmp/msmtp.log
#+end_example
* Multiple Account Setup

=M-x apropros RET gnus-secondary-select-methods=

#+begin_src emacs-lisp
(setq gnus-select-method '(nntp "news.gmane.org"))
#+end_src

#+begin_src emacs-lisp
(setq gnus-secondary-select-methods
    '((nnimap "fastmail"
          ....
          (nnimap-inbox ("INBOX")))
      (nnimap "uber"
          ....
          (nnimap-list-pattern ("INBOX")))
      (nnrss)))
#+end_src

I have nntp, work mail and personal mail, and RSS feeds!
* Sending from Multiple Accounts

Setup Group Parameters

#+begin_src emacs-lisp
(setq gnus-parameters
  '((".*fastmail.*"
     (posting-style
      (address "ryan@whatthefuck.computer")
      (name "Ryan Rix")
      (eval (setq message-sendmail-extra-arguments '("-a" "fastmail")))
      (user-mail-address "ryan@whatthefuck.computer")))
    (".*uber.*"
     (posting-style
      (address "rrix@uber.com")
      (name "Ryan Rix")
      (eval (setq message-sendmail-extra-arguments '("-a" "work")))
      (user-mail-address "rrix@uber.com")))))
#+end_src

* Sending from Multiple Accounts

Setup Group Parameters

Feed the correct arguments in to =msmtp=

#+begin_src emacs-lisp
(defun cg-feed-msmtp ()
  (if (message-mail-p)
      (save-excursion
        (let* ((from
                (save-restriction
                  (message-narrow-to-headers)
                  (message-fetch-field "from")))
               (account
                (cond
                 ((string-match "ryan@whatthefuck.computer" from) "fastmail")
                 ((string-match "rrix@uber.com" from) "work"))))
          (setq message-sendmail-extra-arguments (list '"-a" account))))))

(setq message-sendmail-envelope-from 'header)
(add-hook 'message-send-mail-hook 'cg-feed-msmtp)
#+end_src
* Scoring

- Sort by "how much do you care" not "how recently it showed up"
- Control sorting rules

#+begin_src emacs-lisp
(setq gnus-thread-sort-functions
      '(gnus-thread-sort-by-subject
        gnus-thread-sort-by-date
        gnus-thread-sort-by-total-score))
#+end_src

- Press =L= to create score lowering rules
- Press =I= to create score increasing rules

* Adaptive Scoring

- What if you could do the above automatically? (Gmail "priority inbox")
- You can.

#+begin_src emacs-lisp
(setq gnus-use-adaptive-scoring '(word line))
(setq gnus-adaptive-word-length-limit 5)
(setq gnus-default-adaptive-score-alist
      '((gnus-unread-mark)
        (gnus-ticked-mark (from 4))
        (gnus-dormant-mark (from 5))
        (gnus-del-mark (from -4) (subject -1))
        (gnus-read-mark (from 4) (subject 2))
        (gnus-expirable-mark (from -1) (subject -1))
        (gnus-killed-mark (from -1) (subject -3))
        (gnus-kill-file-mark)
        (gnus-ancient-mark)
        (gnus-low-score-mark)
        (gnus-catchup-mark (from -1) (subject -1))))
#+end_src

* Splitting

Add to your select method:

#+begin_src emacs-lisp
(nnimap-split-fancy (|
(from ".*@nextdoor.com.*" "INBOX.social")
(from ".*@facebookmail.com.*" "INBOX.social")
(from ".*@linkedin.com.*" "INBOX.social")
(from ".*@e.linkedin.com.*" "INBOX.social")
("List-Id" ".*plasma-devel.kde.org.*" "INBOX.kde.plasma")
(any ".*bugzilla_noreply@kde.org.*" "INBOX.kde.bugs")
"INBOX"))
#+end_src

- =M-x apropros nnimap-split-fancy=
* Splitting

- Rich splitting rules
- Not sieve
- Also not procmail

Add to your select method:

#+begin_src emacs-lisp
(nnimap-split-methods 'nnimap-split-fancy)
#+end_src
